
;=============================================================================
;PCX2TXT  2.0  Display Demo
;Copyright (c) 1992 Matthew A. Hudson, All Rights Reserved.
;
;     This program demontrates how to use the text files that PCX2TXT
; generates. It covers CGA 4 color and EGA 16 color. It also demostrates
; the differences in displaying an EGA plane by plane, byte by byte, and
; masked byte by byte picture.
;
;=============================================================================

;======================================
;    		MACROS
;======================================
Outport        MACRO arg1
               mov ax,arg1         ;Values to output.
               out dx,ax           ;Output data.
               ENDM

;======================================
;  			EQUATES
;======================================

;==================
;DOS EQUATES.
;==================
DOS_INT        EQU 21h   ;DOS interrupt call.
GETCHAR        EQU 7     ;DOS get character without echo function call.
OUTSTRING      EQU 9     ;DOS write string function call.
ENDPROGRAM     EQU 4C00h ;DOS terminate program function call.

;==================
;BIOS Video EQUATES.
;==================
VIDEO_INT      EQU 10h   ;BIOS Video interrupt call.
SETPOS         EQU 2     ;BIOS set cursor position function call.
WRITECHAR      EQU 9     ;BIOS write charcter and attribute function call.
TEXTMODE       EQU 3     ;80 x 25 text mode.
CGAMODE        EQU 4     ;BIOS video mode 4 is CGA 320 x 200 4 color.
EGAMODE        EQU 13    ;BIOS video mode 13 is EGA 320 x 200 16 color.

;==================
;MISC. EQUATES.
;==================
CGAMEM         EQU 0b800h ;CGA video memory start.
CGAWIDTH       EQU 80    ;CGA screen width for 320 x 200 4 color mode.
CGAODD         EQU 2000h ;CGA odd row offset.
EGAMEM         EQU 0a000h ;EGA video memory start.
EGAWIDTH       EQU 40    ;EGA screen width for 320 x 200 16 color mode.
MDA            EQU 1     ;returned in Atype if Monocrome present.
HGC            EQU 2     ;returned in Atype if Hercules present.
CGA            EQU 3     ;returned in Atype if Color Graphics present.
EGA            EQU 4     ;returned in Atype if Enhanced Graphics present.
VGA            EQU 5     ;returned in Atype if Video Graphics Array present.

.MODEL SMALL
.CODE
;=============================================================================
;MAIN
;=============================================================================
Main           PROC NEAR
               assume ds:@data
               mov ax,@data
               mov ds,ax           ;set DS to @data.
               call WhichAdapter   ;Determine adapter type.
               cmp BYTE PTR Atype,CGA ;Is adapter CGA or better?
               jge AdapterIsGood      ; y: go display CGA graphics.
               mov ah,OUTSTRING
               mov dx,offset @data:BadAtype ;Load offset of error message.
               int DOS_INT         ;Display error message.
               mov ax,ENDPROGRAM
               int DOS_INT         ;Terminate program.
AdapterIsGood:
               call CGApart        ;Display CGA graphics.
               cmp BYTE PTR Atype,EGA ;EGA card or better?
               jge DoEGAwork          ; y: do EGA stuff.
                                      ; n: tell them why we cannot go on.
               mov ax,TEXTMODE
               int VIDEO_INT       ;Change to text mode.
               mov ah,OUTSTRING
               mov dx,offset @data:EndDemoNoEGA ;Load offset of end message.
               int DOS_INT         ;Display error message.
               mov ax,ENDPROGRAM
               int DOS_INT         ;Terminate program.
DoEGAwork:
               call EGApart
               mov ax,TEXTMODE
               int VIDEO_INT       ;Change to text mode.
               mov ax,ENDPROGRAM
               int DOS_INT         ;Terminate program.
Main           ENDP

;=============================================================================
;CGAPART - Displays a CGA PCx picture that was converted using PCX2TXT.
;=============================================================================
CGApart        PROC NEAR
               mov ax,CGAMEM
               mov es,ax           ;Set ES to CGA screen memory.
               cld
               mov ax,CGAMODE
               int VIDEO_INT       ;Set video mode.

               mov di,(50-9)*CGAWIDTH+32 ;Set screen row pointer to
                                         ; start position.
               mov si,offset @data:CGApic ;Set SI to point to graphic.
               mov dx,34           ;Height of graphic in LINES.
DisplayCGArow:
               mov cx,16           ;Width of graphic in BYTES.
               rep movsb           ;Copy graphic line to screen memory.
               sub di,16           ;Subtract width of graphic from screen
                                   ; row pointer.
               dec dx              ;Decrease line count.
               jz DisplayText      ;Last line complete?
                                   ; y: go display description text.
               cmp di,CGAODD       ; n: is DI pointing to an odd row?
               jae RowEven         ;    y: next row will be even.
               add di,CGAODD       ;    n: next row will be odd.
               jmp DisplayCGArow   ;Display next row.
RowEven:
               sub di,CGAODD       ;Subtract odd row offset.
               add di,CGAWIDTH     ;Point to next even row.
               jmp DisplayCGArow   ;Diplay next row.
DisplayText:
               mov si,offset @data:CGAxpln ;Point to CGA description.
               xor dx,dx           ;Display at 0,0.
               mov bl,3            ;With this attribute. (CGA white)
               call Gprint         ;Display description text.
               call GetAKey        ;Wait for a key before continue.
               ret                 ;Return to caller.
CGApart        ENDP

;=============================================================================
;EGAPART - Displays three EGA PCx pictures using three different modes.
;=============================================================================
EGApart        PROC NEAR
               mov ax,EGAMEM
               mov es,ax           ;Set ES to EGA screen memory.
               mov ax,EGAMODE
               int VIDEO_INT       ;Set video mode.

;======================================
; EGA PLANE BY PLANE - Generally faster
; if you are displaying large pictures.
; Does not preserve background.
;======================================
               mov di,EGAWIDTH*35  ;Set DI to point to start position.
NextppRep:
               push di             ;Save DI for next repition.
               mov si,offset @data:EGApp ;Set SI to point to graphic.
               mov bh,41           ;Height of graphic.
               mov bl,8            ;Width of graphic.
               mov cl,bl           ;Set CX counter.
               mov dx,3c4h         ;EGA Sequencer port number.
WriteNextLine:
               Outport 0102h       ;Set EGA to plane 1.
               repne movsb         ;Write plane.
               mov cl,bl           ;Reset CX counter.
               sub di,cx           ;Reset DI for next plane.

               Outport 0202h       ;Set EGA to plane 2.
               repne movsb         ;Write plane.
               mov cl,bl           ;Reset CX counter.
               sub di,cx           ;Reset DI for next plane.

               Outport 0402h       ;Set EGA to plane 4.
               repne movsb         ;Write plane.
               mov cl,bl           ;Reset CX counter.
               sub di,cx           ;Reset DI for next plane.

               Outport 0802h       ;Set EGA to plane 8.
               repne movsb         ;Write plane.
               mov cl,bl           ;Reset CX counter.
               sub di,cx           ;Reset DI for next line.

               add di,EGAWIDTH     ;Add screen width for next line.
               dec bh              ;Decrease line count.
               jnz WriteNextLine   ;If not last line continue.
               pop di              ;Get DI.
               add di,EGAWIDTH*10  ;Add 10 screen lines.
               dec BYTE PTR Reps   ;Decrease repition count.
               jnz NextppRep       ;Last repition complete?
                                   ; n: go do next.
                                   ; y: do byte by byte.
;======================================
;EGA BYTE BY BYTE - Does not preserve
; background.
;======================================
               mov di,EGAWIDTH*35+16 ;Set DI to point to start position.
NextbbRep:
               push di             ;Save DI for next repition.
               mov si,offset @data:EGAbb ;Set SI to point to graphic.
               xor bx,bx           ;Clear byte counter.
               mov cx,34           ;Height of graphic.
               mov dx,3c4h         ;EGA Sequencer port number.
NextByte:
               Outport 0102h       ;Set EGA to plane 1.
               lodsb               ;Load data.
               mov es:[di+bx],al   ;Write plane 1.

               Outport 0202h       ;Set EGA to plane 2.
               lodsb               ;Load data.
               mov es:[di+bx],al   ;Write plane 2.

               Outport 0402h       ;Set EGA to plane 4.
               lodsb               ;Load data.
               mov es:[di+bx],al   ;Write plane 4.

               Outport 0802h       ;Set EGA to plane 8.
               lodsb               ;Load data.
               mov es:[di+bx],al   ;Write plane 8.

               inc bx              ;Increase byte count.
               cmp bx,8            ;CMP byte count, Width of graphic.
                                   ;Last byte done?
               jne NextByte        ; n: do next byte.
               add di,EGAWIDTH     ; y: add screen width to destination.
               xor bx,bx           ;Clear byte count.
               loop NextByte       ;Loop if not last line.
               pop di              ;Get DI.
               add di,EGAWIDTH*10  ;Add 10 screen lines.
               inc BYTE PTR Reps   ;Increase repition count.
               cmp BYTE PTR Reps,10 ;Last repition complete?
               jne NextbbRep        ; n: go do next.
                                    ; y: do masked byte by byte.
;======================================
;EGA MASKED BYTE BY BYTE - Allows you
; to preserve background.
;======================================
               mov di,EGAWIDTH*35+32 ;Set DI to point to start position.
NextmbbRep:
               push di             ;Save DI for next repition.
               mov si,offset @data:EGAmbb ;Set SI to point to graphic.
               xor bx,bx           ;Clear byte counter.
               mov cx,34           ;Height of graphic.
NextMByte:
               mov dx,3ceh         ;EGA graphics controller port number.
               lodsb               ;Load mask.
               mov ah,8            ;EGA mask byte set address.
               xchg ah,al          ;Put values in correct places.
               out dx,ax           ;Set byte mask.

               mov dx,3c4h         ;EGA sequencer port number.
               Outport 0102h       ;Set EGA to plane 1.
               lodsb               ;Load data.
               xchg es:[di+bx],al  ;Screen must be read before written.

               Outport 0202h       ;Set EGA to plane 2.
               lodsb               ;Load data.
               xchg es:[di+bx],al  ;Screen must be read before written.

               Outport 0402h       ;Set EGA to plane 4.
               lodsb               ;Load data.
               xchg es:[di+bx],al  ;Screen must be read before written.

               Outport 0802h       ;Set EGA to plane 8.
               lodsb               ;Load data.
               xchg es:[di+bx],al  ;Screen must be read before written.

               inc bx              ;Increase byte count.
               cmp bx,8            ;CMP byte count, Width of graphic.
                                   ;Last byte done?
               jne NextMByte       ; n: do next byte.
               add di,EGAWIDTH     ; y: add screen width to destination.
               xor bx,bx           ;Clear byte count.
               loop NextMByte      ;Loop if not last line.
               pop di              ;Get DI.
               add di,EGAWIDTH*10  ;Add 10 screen lines.
               dec BYTE PTR Reps   ;Decrease repition count.
               jnz NextmbbRep      ;Last repition done?
                                   ; n: go do next.
               Outport 0f02h       ; y: set EGA to all planes.
               mov dx,3ceh         ;EGA graphics controller port number.
               Outport 0ff08h      ;Set mask to all ones.

               mov si,offset @data:EGAxpln ;Point to EGA description.
               xor dx,dx           ;Display at 0,0.
               mov bl,15           ;With this attribute. (EGA white)
               call Gprint         ;Display description text.
               call GetAKey        ;Wait for a key before continue.
               ret                 ;Return to caller.
EGApart        ENDP

;=============================================================================
;			MISC. ROUTINES.
;=============================================================================

;======================================
;WHICHADAPTER - determines the type
; of video adapter in system.
; Returns:
;    Atype - holds video adapter type.
;======================================
WhichAdapter   PROC NEAR
               mov ax,1b00h        ;BIOS call 1Bh returns 1 if VGA.
               int VIDEO_INT
               cmp al,1            ;Is this VGA?
               jne TestIfEGA       ; n: check for EGA.
               mov Atype,VGA       ; y: set Atype.
               ret                 ;Return to caller.
TestIfEGA:
               mov ax,40h          ;EGA Information in byte at 40h:87h
               mov es,ax           ;Set ES to 40h
               test BYTE PTR es:[87h],0ffh ;Is it EGA?
               jz TestIfCGAorHGC   ; n: go and check for CGA.
               mov Atype,EGA       ; y: set Atype.
               ret                 ;Return to caller.
TestIfCGAorHGC:
               mov bx,es:10h       ;Get equipment list.
               and bx,30h          ;Bits 5 and 4 show video.
               cmp bx,30h          ;Is it CGA?
               jne ItIsCGA         ; y: set Atype.
               mov cx,8000h        ; n: setup for HGC test.
               mov dx,3bah         ;Bit 7 at port 3BAh will be set
TestIfHGC:                         ; during vertical retrace if HGC.
               in al,dx            ;Get value at port 3BAh.
               test al,80h         ;Is it HGC?
               jnz ItIsHGC         ; y: set Atype.
               loop TestIfHGC      ; n: don't know loop and make sure.
                                   ;If the above loop test does not
                                   ; return a set bit -
               mov Atype,MDA       ;Set Atype to MDA.
               ret                 ;Return to caller.
ItIsHGC:
               mov Atype,HGC       ;Set Atype to HGC.
               ret                 ;Return to caller.
ItIsCGA:
               mov Atype,CGA       ;Set Atype to CGA.
               ret                 ;Return to caller.
WhichAdapter   ENDP

;======================================
;GPRINT - prints characters to screen
; while in graphics mode.
;    ASSUMES VIDEO PAGE IS ZERO!
;    When Calling:
;         DH = starting row.
;         DL = starting column.
;         BL = text attribute.
;         SI = pointer to text.
;======================================
Gprint         PROC NEAR
               mov ColumnHolder,dl ;Store starting column.
               mov ah,SETPOS
               xor bh,bh           ;Set video page to zero.
               int VIDEO_INT       ;Set cursor position.
CharWrite:
               lodsb               ;Load first byte from SI pointer.
               cmp al,'@'          ;Increament to next row?
               je BumpRow          ; y: set for next row.
               cmp al,'$'          ; n: End of text?
               je EndofText        ;    y: end and return to caller.
               mov ah,WRITECHAR    ;    n: display character.
               mov cx,1            ;Display 1 charcter.
               int VIDEO_INT       ;Display character.
               inc dl              ;Increase column pointer.
               mov ah,SETPOS
               int VIDEO_INT       ;Set new cursor position.
               jmp CharWrite       ;Load next character.
BumpRow:
               inc dh              ;Increase row pointer.
               mov dl,ColumnHolder ;Load starting column.
               mov ah,SETPOS
               int VIDEO_INT       ;Set new cursor position.
               jmp CharWrite       ;Load next character.
EndofText:
               ret                 ;Return to caller.
Gprint         ENDP

;======================================
;GETAKEY - Fetches a character from the
; keyboard and discards scancode if an
; extended ASCII character key was
; pressed.
;======================================
GetAKey        PROC NEAR
               mov ah,GETCHAR      ;Get a character from keyboard.
               int DOS_INT
               cmp al,0            ;Is there a scan code?
               jnz NoMoreChars     ; n: return to caller.
               int DOS_INT         ; y: get scan code.
NoMoreChars:
               ret                 ;Return to caller.
GetAKey        ENDP

.DATA
BadAtype       db 'ERROR: Video adapter not CGA or better.$'
EndDemoNoEGA   db 'CGA demo complete - EGA not present program terminated.$'
CGAxpln        db 'CGA 320 x 200 4 color mode demonstration@'
               db 'using graphic converted with PCX2TXT.$'
EGAxpln        db 'EGA 320 x 200 16 color mode demonstra-@'
               db 'tion of Plane by Plane, Byte by Byte,@'
               db 'and Masked Byte by Byte.$'
Atype          db ?      ;Memory area to store Video Adapter Type.
ColumnHolder   db ?      ;Gprint column holder.
Reps           db 10     ;Number of times to repeat process.
CGApic         LABEL DATAPTR
               INCLUDE CGAPIC.HEX
EGApp          LABEL DATAPTR
               INCLUDE EGAPP.HEX
EGAbb          LABEL DATAPTR
               INCLUDE EGABB.HEX
EGAmbb         LABEL DATAPTR
               INCLUDE EGAMBB.HEX

.STACK         200h
               END Main